% Perli.m
% calculates adjustment dynamics for the two-sector growth model
% uses perlipol.m, ode45mod.m
% References:
% Benhabib and Perli (1994) 
% Brunner and Strulik (1998, 2001)

clear all   
global sig rho b gam del A psi phi xi eta xstar ustar qstar

%********* Initialize Parameters **************************************
sig=0.15; %inverse of int subst ela
rho=0.0505; %time preference rate
b=0.65; %capital share
gam=0.1; %h-cap spillover param 	
del=0.05; %productivity param in edu
A=1;

psi=del*(1-b+gam)/(b-1); phi=b/sig-1;
xi=rho/sig; eta=del*(b-gam)/b;

%************ Calculate steady state values ***************************
ustar=1-(1-b)*(rho-del)/(del*(gam-sig*(1-b+gam)));
xstar=((b*xi-del*(1-b+gam)-del*(b-gam)*ustar)/(A*b*phi))^(1/(b-1))*ustar;
qstar=eta*ustar+del*(1-b+gam)/b;

% *********** Calculate Eigensystem ***********************************
j11=A*(b-1)*xstar^(b-1)*ustar^(1-b); j12=-xstar/ustar*(j11+psi*ustar);
j31=j11*phi*qstar/xstar; j32=-j11*phi*qstar/ustar;

J=[j11 j12 -xstar; 0 eta*ustar -ustar; j31 j32 qstar];
[eigvec,eigval]=eig(J); v1=real(eigvec(:,1)); v2=imag(eigvec(:,1));
%v1=1/norm(v1)*v1; v2=1/norm(v2)*v2;

%**** Generating Initial Values on a Cycle Around the steady-state ****
epsilon=10^(-7);    % Deviation from the steady-state  
for i=0:0.1:2
i
theta=i*pi;
ystar=[xstar;ustar;qstar];
ystart=ystar+epsilon*sin(theta)*v1+epsilon*cos(theta)*v2;

% ***************** Backward Integration *******************************
ufinal=0.99;        % Terminal Condition for u
[t,pol] = ode45mod('perlipol',ystart,ufinal,2,10^(-6),0);
pol=pol(length(t):-1:1,:);
x=pol(:,1);u=pol(:,2);q=pol(:,3);

%******************* Plotting Routine ************
set(gcf,'PaperUnits', 'centimeters','PaperType','a4letter','Units','centimeters','PaperPosition', [0.5 1.5 15 7.5],'Position',[0.5 1.5 15 7.5],'Color','white');
plot3(x,u,q,'b-')
hold on
set(gca,'FontName','Times','FontSize',10);
end %for i

xlabel('{\it x}','FontSize',12), ylabel('{\it u}','FontSize',12),
zlabel('{\it q}','FontSize',12),
axis([-Inf +Inf -Inf +Inf -Inf +Inf])
view(-30,30)

